/*****************************************************************************
*
* File              : TWI_vnc_slave.c
*							 01.05.09, TAB = 3
* AppNote           : AVR311 / 315
*
*
* nderungen        : Enthlt die Fuktionalit eines Slaves.
*							 Luft als (vereinfachter) Slave unabhngig von main():
*							 Es sind nur die Empfangsroutinen aktiv.
*
****************************************************************************/

#include <avr/interrupt.h>              
#include <avr/io.h>
#include <avr/eeprom.h>
#include <inttypes.h>
#include "TWI_df_slave.h"
#include "global.h"
#include "at45dbx81.h"

// ----------------------------------------------------------------------------
// Initailisierung fr den Slave Modus
void TWI_init(void)
{
	TWAR = TWI_SLA_ADR;
   TWBR = TWI_TWBR;                             // Set bit rate register (Baudrate). Defined in header file.
// TWSR = TWI_TWPS;                             // Not used. Driver presumes prescaler to be 00.
//	TWDR = 0xFF;                                 // Default content = SDA released.
/*	TWCR = (1<<TWEN)|                            // TWI Interface enabled.
          (1<<TWIE)|(1<<TWINT)|                 // Enable TWI Interupt and clear the flag.
          (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|      // Prepare to ACK next time the Slave is addressed.
          (0<<TWWC); */ 
	TWCR = 0;
}

/****************************************************************************
This function is the Interrupt Service Routine (ISR), and called when the TWI interrupt is triggered;
that is whenever a TWI event has occurred. This function should not be called directly from the main
application.
****************************************************************************/

ISR(TWI_vect)
{
	switch (TWSR)
	{
// TWI Slave Transmitter status codes				// ###############################################

   case TWI_STX_ADR_ACK:            				// Own SLA+R has been received; ACK has been returned

// der Slave hat ein Byte gesendet, der Master mit ACK quittiert
   case TWI_STX_DATA_ACK:           				// Data byte in TWDR has been transmitted; ACK has been received
																// Es werden solange Daten gesendet, bis der Master mit NACK beendet
																// damit kann der Pointer aus dem Array hinauslaufen. 
								
		TWDR = 0x55; 										// als Standardantwort - falls jemand fragt ...
		
      TWCR =(1<<TWEN)|                         	// TWI Interface enabled
            (1<<TWIE)|(1<<TWINT)|              	// Enable TWI Interupt and clear the flag to send byte
            (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|   	// Send ACK
            (0<<TWWC);                         	//  
      break;


// TWI Slave Receiver status codes					// ###############################################

// der Slave ist im read-Modus und hat gerade seine eigene Adresse empfangen
// eigene Slave-Adresse + write empfangen und mit ACK quittiert
   case TWI_SRX_ADR_ACK: 								// Own SLA+W has been received ACK has been returned
		TWCR = (1<<TWEN)|                         // TWI Interface enabled
             (1<<TWIE)|(1<<TWINT)|              // Enable TWI Interupt and clear the flag to send byte
             (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|   // Send ACK after next reception
             (0<<TWWC);                         // 
		break;
			
// Slave im read Modus, Byte empfangen und mit ACK quittiert, nun das Byte ablegen
   case TWI_SRX_ADR_DATA_ACK:       				// Previously addressed with own SLA+W; data has been received; 
																// ACK has been returned
	
		DF_write_byte(TWDR);								// das empfangene Byte direkt wegschreiben

		TWCR = (1<<TWEN)|                         // TWI Interface enabled
             (1<<TWIE)|(1<<TWINT)|              // Enable TWI Interupt and clear the flag to send byte
             (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|   // Send ACK after next reception
             (0<<TWWC);                         //  
		break;

// case TWI_STX_DATA_NACK:          				// Data byte in TWDR has been transmitted; NACK has been received. 
																// I.e. this could be the end of the transmission.
//	case TWI_STX_ADR_ACK_M_ARB_LOST: 				// Arbitration lost in SLA+R/W as Master; 
																// own SLA+R has been received; ACK has been returned
// case TWI_STX_DATA_ACK_LAST_BYTE: 				// Last data byte in TWDR has been transmitted (TWEA = 0); 
																// ACK has been received
// case TWI_SRX_ADR_DATA_NACK:      				// Previously addressed with own SLA+W; data has been received; 
																// NOT ACK has been returned
// case TWI_SRX_GEN_DATA_NACK:      				// Previously addressed with general call; 
																// data has been received; NOT ACK has been returned
// case TWI_SRX_GEN_ACK:            				// General call address has been received; ACK has been returned

// case TWI_SRX_GEN_DATA_ACK:       				// Previously addressed with general call; 
																// data has been received; ACK has been returned
// case TWI_SRX_ADR_ACK_M_ARB_LOST: 				// Arbitration lost in SLA+R/W as Master; 
																// own SLA+W has been received; ACK has been returned 
// case TWI_SRX_GEN_ACK_M_ARB_LOST: 				// Arbitration lost in SLA+R/W as Master; 
																// General call address has been received; ACK has been returned
// case TWI_SRX_STOP_RESTART:       				// A STOP condition or repeated START condition has been received 
																// while still addressed as Slave 
//	case TWI_NO_STATE 									// No relevant state information available; TWINT = 0
// case TWI_BUS_ERROR:         						// Bus error due to an illegal START or STOP condition

// im Slave-Modus laufen alle anderen Flle ber default:
   default:     
     
		TWCR = (1<<TWEN)|                         // TWI Interface enabled
             (1<<TWIE)|(1<<TWINT)|              // Enable TWI Interupt and clear the flag to send byte
             (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|   // Send ACK after next reception
             (0<<TWWC);                      
		break; 
	}
}

// -- [ eof ] -----------------------------------------------------------------