	// --- [ logger.c ] -----------------------------------------------------------
//
//       ergnzt: Whrend der Aufnahme deaktiviert die Taste PLAY die LEDs.
//                Der Status des Chips wird bei der Ausgabe mit ausgedruckt.
//                Die empfangenen Bytes werden direkt und einzeln in den 
//                Buffer des Dataflash geschrieben, weniger SRAM erforderlich

#define EXTERN

#include <avr/io.h>
#include <string.h>	
#include <stdlib.h>
#include <stdint.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include "rs232.h"
#include "spi_hard.h"
#include "global.h"
#include "TWI_df_slave.h"
#include "at45dbx81.h"

// --- public - Variablen -----------------------------------------------------

#define LOG_TWI									// via TWI   loggen
#define LOG_RS232									// via RS232 loggen

void record(void);
void play(void);

// ----------------------------------------------------------------------------
int main(void)
{
CLKPR = (1<<CLKPCE);								// Clockprescale = 1
CLKPR = 0; 
														// Strom sparen, Timer2/1 und ADC abschalten
PRR = (1<<PRTIM2 | 1<<PRTIM1 | 0<<PRTIM0 | 0<<PRSPI | 0<<PRUSART0 | 1<<PRADC);

// Der Timer0 wird im Clear Timer on Compare Match Modus betrieben
// Sobald der TCTN0 den Wert in OCR0A berschreiten, wird der TCNT0
// auf 0 zurckgesetzt und eine ISR angesprungen.
// Hier wird allgemein die Zeit aktualisiert sowie die Variable VNC_timeout decrementiert

TCCR0A  = (1<<WGM01);							// CTC-Modus
TCCR0B  = (1<<CS02 | 1<< CS00);				// prescale 1024
TIMSK0  = (1<<OCIE0A);							// enable COMPA
OCR0A   = (250 - 1);

LED_DDR  = (1<<LED1 | 1<<LED2 | 1<<LED3);	// Port fr LED als Ausgang
LED1_ON;												// alle LEDs ein
LED2_ON;
LED3_ON;
														// fr einen Taster zum Starten/Stoppen des Loggens/ndern der Zeit
KEY_PORT |= ( 1<<KEY_START | 1<<KEY_PLAY);// Pullups setzen

RS232_init();										// RS232 initialisieren
SPI_init();
TWI_init();
//TWI_ENABLE;										// nur zum Debuggen

asm volatile ("sei");							// fr Timer und auch RX/TWI/Timer erforderlich

SPI_init();											// Initialisierung, danach braucht der DF 20ms

LED_timeout = 15;									// 1/2 Sekunde warten
while (LED_timeout);

if (!(DF_device_ready()))						// wenn dann das DF nicht bereit ist
	{
	LED1_OFF;
	LED3_OFF;
	while(1)											// dann liegt wohl ein Problem vor
		{
		LED_timeout = 7;
		while (LED_timeout);						// nur LED2 toggelt
		LED2_TOG;
		}
	}

// --- Hauptschleife ----------------------------------------------------------
//     Vorbereitung: 
//		 das Dateisystem durchsuchen, die letzte File-ID bestimmen (file_id)
//     die nchste frei Page (ddf_next_page) finden,
//     die Anzahl der freien und belegten Pages ermitteln (for fun)
//
//     diese Phase durch blinkende LED markieren
//
//     danach Warten auf Startzeichen - Taste START oder PLAY

DF_check();											// hier wird die nchste freie Page ermittelt

// DF_file_test();								// druckt alle Files auf dem DataFlash aus

LED1_OFF;											
LED2_ON;												// Nur LED2 ist eingeschaltet
LED3_OFF;							

while(1)
	{	
	if (KEY_START_PRESSED)						// wenn die START-Taste gedrckt wird
		{
		LED_timeout = 10;							// 1/3 Sekunde Wartezeit
		while(LED_timeout);						// warten
		while((KEY_START_PRESSED) && (KEY_PLAY_RELEASED));	// warten bis die Start-Taste losgelassen wird
														// oder zustzlich die Play-Taste gedrckt wird
														
		if ((KEY_PLAY_RELEASED))				// wenn ist die Starttaste losgelassen worden
			{
			record();								// wenn immer noch alleine gedrckt, dann zur Aufnahme springen

			LED1_OFF;	
			LED2_ON;									// Nur LED2 ist eingeschaltet
			LED3_OFF;
			}
		}

	else if (KEY_PLAY_PRESSED)					// wenn die PLAY-Taste gedrckt wird
		{
		LED_timeout = 10;							// 1/3 Sekunde Wartezeit
		while(LED_timeout);						// warten
		while((KEY_PLAY_PRESSED) && (KEY_START_RELEASED));	// warten bis die Play-Taste losgelassen wird
														// oder zustzlich die Starttaste gedrckt wird
 
		if (KEY_START_RELEASED)					// dann wurde die Play-Taste losgelassen 
			{
			play();									// wenn immer noch alleine gedrckt, dann zur Wiedergabe springen
			
			LED1_OFF;	
			LED2_ON;									// Nur LED2 ist eingeschaltet	
			LED3_OFF;
			}
		}
		

	if ((KEY_PLAY_PRESSED) && (KEY_START_PRESSED)) // wenn START- und PLAY-Taste gleichzeitig gedrckt werden
		{

		LED_timeout = 50;							// noch einmal 5/3 Sekunde Wartezeit insgesamt also 2 Sekunden
		while(LED_timeout);						// warten
		
		if ((KEY_PLAY_PRESSED) && (KEY_START_PRESSED)) // wenn die Tasten immer noch gedrckt sind
			{
			DF_erase_chip();						// den ganzen Chip lschen
			DF_check();								// und neu die Startdaten ermitteln
			
			LED1_OFF;
			LED2_ON;									// Nur LED2 ist eingeschaltet	
			LED3_OFF;
			}
		}
	}
}

// ----------------------------------------------------------------------------
// hier werden Daten aufgezeichnet
// die LED1 toggelt langsam - aber stetig
// die LED2 toggelt nach jedem geschriebenen Datensatz
// Beendet wird durch Drcken der START-Taste
void record(void)
{
uint8_t i;
df_byte_adr = 0;									// Byte_Adresse im DataFlash-Buffer
buffer_idx = 0;									// schreiben in Buffer 1
bytes_received[0] = 0;							// Zhler fr empfangene Bytes in Buffer1
//bytes_received[1] = 0;						// Zhler fr empfangene Bytes in Buffer2

file_page_cnt = 1;								// Zhler fr die Anzahl der Pages
t.time32 = 0;										// die Zeitzhlung zurcksetzen
df_file_id++;										// das ist die nchste File-ID
df_file_id &= 0x7FFF;							// Bit.7 im HIgh-Byte maskieren
														// am gesetzten Bit.7 wird eine freie/gelschte Page erkannt
#ifdef LOG_RS232
	RX_ENABLE;										// Datenannahme via RX  enablen 
#endif
#ifdef LOG_TWI 
	TWI_ENABLE; 									// Datenannahme via TWI enablen
#endif

LED1_ON;												// LED1 ist an, LED2 toggelt permanent,
														// LED3 toggled nach dem Schreiben einer Page
// die ID incrementieren
// den Pagecounter zurcksetzen

session_pages = 0;								// Counter fr die geschriebenen Pages der Sitzung
														// wenn die Gesamtblockzahl erreicht ist,
														// dann kann weitere Speicherung beendet werden, um nicht
														// den Anfang der Aufzeichnung zu berschreiben
// --- innere Hauptschleife ----------------------------------------------------------

while(1)												// Schleife fr den Empfang von Daten
	{
	if (!(LED_timeout)) 							// nach 1 Sekunde ist VNC_timeout vom Timer auf 0 decrementiert
		{
		LED_timeout = 15;							// LED1 blinkt mit 1 Hz whrend des Loggens
		LED2_TOG;									
		}
			
	if (buffer_full_flag)						// wenn der Puffer voll ist
		{
		buffer_full_flag = FALSE;
		i = (buffer_idx ^1);						// die Daten stehen im anderen Buffer
		DF_write_header(i);						// 
		LED3_TOG;
		}

														// Wenn die START-Taste bettigt wird, dann wird die geffnete Datei geschlossen 
	if (KEY_START_PRESSED)						// Datei schieen 
		{
#ifdef LOG_RS232
		RX_DISABLE;									// Datenannahme via RX  abschalten 
#endif
#ifdef LOG_TWI 
		TWI_DISABLE; 								// Datenannahme via TWI abschalten
#endif
														// steht rx_idx auf 0, dann hat es gerade einen vollen Puffer
														// gegeben, die Daten sind gesichert, aber kein weiteres
														// Byte ist empfangen worden, daher
		if (bytes_received[buffer_idx])		// nur wenn Bytes im Puffer sind, dann wird auch gespeichert
			{
			DF_fill_buffer();
			DF_write_header(buffer_idx);		// als Parameter wird bergeben, welcher Buffer zu schreiben ist
			}

		LED_DDR |= (1<<LED1 | 1<<LED2 | 1<<LED3); // falls die LEDs abgeschaltet waren
		LED_timeout = 3;
		while (LED_timeout);
		while(KEY_START_PRESSED);				// warten bis die Taste losgelassen wird
		
		return;
		}
	
	if (KEY_PLAY_PRESSED)						// wenn KEY_PLAY whrend der Aufnahme gedrckt wird
		{												// dann werden die LEDs an/abgeschaltet
		LED_timeout = 3;
		while (LED_timeout);
		while (KEY_PLAY_PRESSED);
		
		if (LED_DDR & (1<<LED1 | 1<<LED2 | 1<<LED3)) LED_DDR &= ~(1<<LED1 | 1<<LED2 | 1<<LED3); else LED_DDR |= (1<<LED1 | 1<<LED2 | 1<<LED3);
		}	
	}
}

// ----------------------------------------------------------------------------
// spielt die letzte Aufnahme ab und sendet sie via RS232 ab.
// Nach Abschluss der bertragung wartet das Programm auf Tastenbettigung:
// Ein erneutes Drcken der PLAY-Taste gibt die vorausgehende (ltere) Datei aus
// - bis keine Datei mehr gefunden wird
// Ein Bettigen der START-Taste springt ins Hauptmenue zurck.
void play(void)
{
uint16_t next_page;

RX_ENABLE;											// die Schnittstelle enablen fr die Ausgabe
														// df_next_page zeigt auf die nchste frei Page
next_page  = df_next_page - 1;				// den Wert kopieren, die Variable df_next_page bleibt erhalten

LED1_OFF;											// LED1 aus
														// LED2 blinkt (aus Unterprogramm)
LED3_ON;												// LED3 auf Dauerlicht

while(1)
	{
	next_page &= PAGE_MASK;	
	next_page = DF_print_file(next_page);	// diese Page drucken
														// die Funktion liefert die Adresse der letzten Page
														// der vorausgehenden Datei zurck
	LED2_OFF;										// Dauerlicht an LED3 zeigt Ende der Ausgabe an

	if (next_page > PAGE_MASK)	return;		// beenden, wenn sich ein Fehler/keine weitere Datei ereignet hat

														// warten, bis eine der beiden Tasten bettigt wird
	while (KEY_START_RELEASED && KEY_PLAY_RELEASED);

	LED_timeout = 2;
	while (LED_timeout);

	if (KEY_START_PRESSED) 						// wenn die Starttaste gedrckt wurde, dann zurck zur Hauptebene	
		{
		RX_DISABLE;
		while(KEY_START_PRESSED);				// gleich wieder ein
		return;
		}

// wenn die Play-Taste gedrckt war, dann die nchste Datei ausgeben
	while(KEY_PLAY_PRESSED);					// warten bis die Play-Taste losgelassen wird
	}
}


// ----------------------------------------------------------------------------
// TIMER0_OVF:  3.686400 MHZ / (1024 * 120) -> 33ms  Zhlt VNC_timeout herunter auf 0
// als Grundlage fr die Zeitzhlung muss ein Takt gewhlt werden, aus dem sich
// der Sekundentakt ableiten lsst: Die ISR wird 30x /Sekunde aufgerufen
ISR(TIMER0_COMPA_vect)
{
static uint8_t tick = 30; 
tick--;

if(LED_timeout) LED_timeout--;				// timeout runterzhlen
if (!(tick))										// wenn 1 Sekunde um ist (tick = 30)
	{												
	tick = 30;
	t.time32++;										// die Zeit im Sekundentakt incrementieren
	}
}


// --- [ eof ] ----------------------------------------------------------------

